from functools import wraps
from flask import request
from werkzeug.exceptions import Unauthorized

from flaschengeist import logger
from flaschengeist.controller import sessionController


def login_required(permission=None):
    """Decorator use to make a route only accessible by logged in users.
        Sets ``current_session`` into kwargs of wrapped function with session identified by Authorization header.

    Attributes:
        permission: Optional permission needed for this route

    Returns:
        Wrapped function with login (and permission) guard
    """
    def wrap(func):
        @wraps(func)
        def wrapped_f(*args, **kwargs):
            token = list(filter(None, request.headers.get("Authorization").split(" ")))[-1]
            session = sessionController.validate_token(token, request.user_agent, permission)
            if session:
                kwargs["current_session"] = session
                logger.debug("token {{ {} }} is valid".format(token))
                return func(*args, **kwargs)
            else:
                logger.info("token {{ {} }} is not valid".format(token))
                raise Unauthorized

        return wrapped_f

    return wrap
